#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2002 Free Software Foundation
#
# FILE:
# GFBlock.py
#
# DESCRIPTION:
#
# NOTES:
#
# HISTORY:
#

from gnue.forms.GFObjects.GFDataSource import GFDataSource

from gnue.common import GDebug
from gnue.common import GConditions
from GFContainer import GFContainer
from gnue.common import events

import string

# These should really go somewhere else
TRUE = 1
FALSE = 0


############################################################
# GFBlock
#
#
class GFBlock(GFContainer, events.EventAware):
  def __init__(self, parent=None):
    GFContainer.__init__(self, parent, 'GFBlock')

    self.mode = 'normal'

    self._inits = [self.initialize]

    #self._datasource = None
    self._resultSet = None
    self._dataSourceLink = None
    self._entryList = []

    self._currentRecord = 0
    self._recordCount = 0
    self._queryDefaults = {}
    self._queryValues = {}
    self._lastQueryValues = {}
    self._autocreate =  int(gConfig('autocreate'))
    self._gap = 0
    self._rows = 1

    #
    # Trigger exposure
    #
    self._validTriggers = {
                  'ON-SWITCH':      'On-Switch',
                  'ON-NEWRECORD':   'On-NewRecord',
                  'PRE-COMMIT':     'Pre-Commit',
                  'POST-COMMIT':    'Post-Commit',
                  'POST-QUERY':     'Post-Query', 
                  'PRE-MODIFY':     'Pre-Modify',
                  'PRE-INSERT':     'Pre-Insert',
                  'PRE-DELETE':     'Pre-Delete',
                  'PRE-UPDATE':     'Pre-Update',
                  'PRE-FOCUSOUT':   'Pre-FocusOut',
                  'POST-FOCUSOUT':  'Post-FocusOut',
                  'PRE-FOCUSIN':    'Pre-FocusIn',
                  'POST-FOCUSIN':   'Post-FocusIn',}


    self._triggerGlobal = 1
    self._triggerFunctions={'clear':{'function':self.processClear},
                            'gotoRecord':{'function':self.jumpRecord},
                            'newRecord':{'function':self.newRecord},
                            'nextRecord':{'function':self.nextRecord},
                            'prevRecord':{'function':self.prevRecord},
                            'deleteRecord':{'function':self.deleteRecord},
                            }

    self._triggerProperties={'parent':  {'get':self.getParent}}

  def __getitem__(self, index):

    cnt = self._resultSet.getRecordCount() - 1
    if index > cnt:
      raise IndexError

    self.jumpRecord(index)

    if cnt == index and self.isEmpty():
      print "Bad, bad, bad"
      raise IndexError

    return self

  def _buildObject(self):

    if hasattr(self, 'rows'):
      self._rows = self.rows

    if hasattr(self, 'rowSpacer'):
      self._gap = self.rowSpacer

    if hasattr(self,'datasource'):
      self.datasource = string.lower(self.datasource)

    self.walk(self.__setChildRowSettings)
    return GFContainer._buildObject(self)

  #
  # Primary initialization
  #
  def initialize(self):
    self._form = form = self.findParentOfType('GFForm')
    self._page = self.findParentOfType('GFPage')

    form._blockList.append(self)

    # Initialize our events system
    events.EventAware.__init__(self, self._form._app.eventController)

    # Get all focusable items, ordered correctly
    self._focusOrder = self.getFocusOrder()

    # Find all GFEntry's
    for object in self._focusOrder:
      if object._type == 'GFEntry':
        self._entryList.append(object)

    # Create a stub/non-bound datasource if we aren't bound to one
    if not hasattr(self,'datasource') or not self.datasource:
      ds = GFDataSource(self)
      self.datasource = ds.name = "dts_%s" % self
      self._form._datasourceDictionary[ds.name] = ds
      ds._buildObject()
      ds.phaseInit()

    self._dataSourceLink = self._form._datasourceDictionary[self.datasource]

    # We will monitor our own resultSet changes
    self._dataSourceLink.registerResultSetListener(self._loadResultSet)


  def __setChildRowSettings(self, object):
    if hasattr(object,'rows'):
      object._rows = object.rows
    else:
      object._rows = self._rows

    if hasattr(object,'rowSpacer'):
      object._gap = object.rowSpacer
    else:
      object._gap = self._gap


  #
  # isSaved
  #
  # returns a positive value if the datasource the block is associated
  # with is saved
  #
  def isSaved(self):
    return self._resultSet == None or not self._resultSet.isPending()

  #
  # deleteRecord
  #
  # Doesn't really delete the record but marks it for
  # deletion during next commit
  #
  def deleteRecord(self):
    self._resultSet.current.delete()

  #
  # Load and process a resultset
  #
  def _loadResultSet(self, resultSet):
    self._resultSet = resultSet
    self._currentRecord = -1
    self._recordCount = 0
    resultSet._block = self

    if self._resultSet.firstRecord():
      self.switchRecord(0)
      self._recordCount = self._resultSet.getRecordCount()
    else:
      # TODO: This means no results were returned from a query.
      # TODO: It probably shouldn't create a new record :)  Display a message?
      self.newRecord()

  #
  # isEmpty()
  #
  def isEmpty(self):
    return self._resultSet.current.isEmpty()

  #
  #
  # Moves the proper record into editing position
  #
  def switchRecord(self, adjustment):
    self._currentRecord = self._resultSet.getRecordNumber()
    for entry in self._entryList:
      entry.recalculateVisible( adjustment, self._currentRecord, self._recordCount)
      self._form.updateUIEntry(entry)

    if adjustment <> 0:
      self.processTrigger('ON-SWITCH')

  #
  # newRecord
  #
  # Adds a record to the current records in memory
  #
  def newRecord(self):
    if self._resultSet.insertRecord():
      self._recordCount = self._resultSet.getRecordCount()
      self.switchRecord(1)
      self.processTrigger('ON-NEWRECORD')

  def nextRecord(self):
    if self._resultSet.nextRecord():
      self._recordCount = self._resultSet.getRecordCount()
      self.switchRecord(1)
    elif self._autocreate and not self.isEmpty() and not self.restrictInsert:
      self.newRecord()

  def lastRecord(self):
    self._resultSet.lastRecord()
    self.switchRecord(0)

  def firstRecord(self):
    self._resultSet.firstRecord()
    self.switchRecord(0)

  def prevRecord(self):
    if self._resultSet.prevRecord():
      self.switchRecord(-1)

  def jumpRecord(self, recordNumber):
    # If recordNumber is negative, move relative to last record
    if recordNumber < 0:
      recordNumber += self._resultSet.getRecordCount()
    if recordNumber < 0: 
      raise "Invalid record number"
    if not self._resultSet.setRecord(recordNumber):
      self._resultSet.lastRecord()
    jump = self._resultSet.getRecordNumber() - self._currentRecord
    self._currentRecord = self._resultSet.getRecordNumber()
    self.switchRecord(jump)

  #
  # processCommit
  #
  def processCommit(self):
    GDebug.printMesg(1, "processing commit on block %s"%self.name,1)
    self.mode='commit'

    self._resultSet.setRecord(self._precommitRecord)

    if not self._dataSourceLink.hasMaster():
      self._resultSet.post()
      self._dataSourceLink._dataObject.commit()

    self._recordCount = self._resultSet.getRecordCount()

    # If all our records were deleted, create an empty record
    if not self._recordCount:
      self.newRecord()
    else:
      self.jumpRecord(self._resultSet.getRecordNumber())

    self.mode='normal'


  #
  # processClear
  #
  def processClear(self):
    self._dataSourceLink._dataObject.rollback()
    if not self._dataSourceLink.hasMaster():
      self._dataSourceLink.createEmptyResultSet()
    else:
      self._dataSourceLink.createEmptyResultSet(masterRecordSet=self._resultSet._masterRecordSet)
    self._currentRecord = 0
    self._recordCount = 0
    self.switchRecord(0)


  #
  # processRollback
  #
  # if recovering=0, then the user requested a rollback
  # if recovering=1, then a commit or such failed and we need to clean up
  # (but not lose state information)
  #
  def processRollback(self, recovering=0):
    if not recovering:
      self._currentRecord = 0
      self._recordCount = 0
      self._dataSourceLink.createEmptyResultSet()
    self._dataSourceLink._dataObject.rollback()
    self.switchRecord(0)

  #
  # initQuery and processQuery
  #
  def initQuery(self):

    # If Enter-Query is hit once, enter query mode
    # If Enter-Query is hit twice, bring back conditions from last query.
    # If Enter-Query is hit thrice, cancel the query and go into normal mode.

    if self.mode != 'query':
      for block in self._form._blockList:
        block.mode = 'query'
        block._query2 = int(gConfig("RememberLastQuery"))
        block._queryValues = {}
        block._queryValues.update(self._queryDefaults)
        block.switchRecord(0)
    elif self._query2:
      for block in self._form._blockList:
        block._query2 = 0
        block._queryValues = {}
        block._queryValues.update(block._lastQueryValues)
        block.switchRecord(0)
    else:
      for block in self._form._blockList:
        block.mode = 'normal'
        block.switchRecord(0)

                
  def processQuery(self):
    # Set the maxList to a single master block
    # as a placeholder
    maxList = [self._dataSourceLink._dataObject]
    while maxList[0]._masterObject:
      maxList = [maxList[0]._masterObject]

    # Find the longest master/detail chain that
    # contains query values.  This will become
    # the chain that is queried
    for block in self._form._blockList:
      if block._queryValues.keys():
        templist = [block._dataSourceLink._dataObject]
        while templist[-1]._masterObject:
          templist.append(templist[-1]._masterObject)
        if len(maxList) < len(templist): maxList = templist

    # Store block states
    for block in self._form._blockList:
      block.mode = 'normal'
      block._lastQueryValues = {}
      block._lastQueryValues.update(block._queryValues)

    # graft in the sloppy query stuff if needed    
    for dataobject in maxList:
      for block in self._form._blockList:
        if dataobject == block._dataSourceLink._dataObject:
          break                                            
      for entry in block._entryList:
        if hasattr(entry,'sloppyQuery') and block._queryValues.has_key(entry):
          block._queryValues[entry] = "%"+ string.join(list(block._queryValues[entry]),"%")+"%"

    # Build the sql required by the detail blocks
    SQL = ""
    for dataobject in maxList[:-1]:
      for block in self._form._blockList:
        if dataobject == block._dataSourceLink._dataObject:
          break                                            
      conditions = _generateConditional(block)
      SQL = self._dataSourceLink.getQueryString(conditions,1,SQL)

    for block in self._form._blockList:
     if maxList[-1] == block._dataSourceLink._dataObject:
      break
    rootBlock = block

    conditions = _generateConditional(rootBlock)
    rootBlock._dataSourceLink.createResultSet(conditions, sql=SQL)     
      
    rootBlock._recordCount = rootBlock._resultSet.getRecordCount()

    for block in self._form._blockList:
      block.processTrigger('POST-QUERY')


def _generateConditional(block):
    conditionLike = {}
    conditionEq = {}
    # Get all the user-supplied parameters from the entry widgets
    for entry in block._queryValues.keys():
        if entry._bound and entry.isQueryable() and len(str(block._queryValues[entry])):
          if entry.typecast == 'text':
            GDebug.printMesg(5,'Adding conditional LIKE (%s like %s)' % (entry.field, block._queryValues[entry]))
            conditionLike[entry.field] = block._queryValues[entry]
          else:
            GDebug.printMesg(5,'Adding conditional AND (%s=%s)' % (entry.field, block._queryValues[entry]))
            conditionEq[entry.field] = block._queryValues[entry]

    if len(conditionLike.keys()) and len(conditionEq.keys()): 
        GDebug.printMesg(5,'Combining like w/and (%s, %s)' % (conditionLike, conditionEq))
        conditions = GConditions.combineConditions( \
            GConditions.buildConditionFromDict(conditionLike, GConditions.GClike), \
            GConditions.buildConditionFromDict(conditionEq, GConditions.GCeq) )

    elif len(conditionLike.keys()): 
        conditions = GConditions.buildConditionFromDict(conditionLike, GConditions.GClike)

    elif len(conditionEq.keys()):
        conditions = GConditions.buildConditionFromDict(conditionEq, GConditions.GCeq)

    else: 
        conditions = {}

    return conditions




