#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2002 Free Software Foundation
#
# FILE:
# GFParser.py
#
# DESCRIPTION:
# Class that contains a sax based xml processor for GNUE forms
#
# NOTES:
#

from gnue.common import GDataSource, GParser, GMenu

import copy, types
from gnue.common import GTypecast, GTrigger





########
########  Please keep this file neat !!!
########






#######################################################
# This method loads a form from an XML file and returns
# a GFForm object.  If initialize is 1 (default), then
# the form is initialized and ready to go.
#
# (initialize=0 is currently not used -- will probably
#  be used in the Forms Designer package where we will
#  not want the loaded form to connect to databases, etc)
#######################################################

def loadFile(buffer, app, initialize=1):
  return GParser.loadXMLObject (buffer, xmlFormsHandler, 'GFForm', 'form',
           initialize, attributes={"_app": app,
                               "_parameters": app._parameters, 
                               "_connections": app.connections })


xmlElements = None


def getXMLelements():

  global xmlElements

  if xmlElements == None:
    from gnue.forms import GFObjects, GFLibrary, GFForm

    xmlElements = {
      'form': {
         'BaseClass': GFForm.GFForm,
         'Required': 1,
         'SingleInstance': 1,
         'Attributes': {
            'title': {
               'Typecast': GTypecast.text,
               'Default': 'Untitled Form',
               'Description': 'The title of the form. Will be displayed on About Screen.' },
            'readonly': {
               'Typecast': GTypecast.boolean,
               'Default': 0,
               'Description': 'TODO' },
            'tabbed': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                  'left': {},
                  'right': {},
                  'bottom': {},
                  'top': {} },
               'Default': "",
               'Description': 'Allows a form to convert it\'s pages as notebook tabs. Allowed values are {left}, {right}, {bottom}, {top}.' },
            'width': {
               'Typecast': GTypecast.whole,
               'Default': 40,
               'Description': 'The width of the object in text columns.' },
            'height': {
               'Typecast': GTypecast.whole,
               'Default': 12,
               'Description': 'The height of the object in text rows.' } },
         'ParentTags': None,
         'Description': 'TODO' },

      'database': {
         'BaseClass': GFObjects.GFDatabase,
         'Attributes': {
            'name': {
               'Required': 1,
               'Unique': 1,
               'Typecast': GTypecast.name,
               'Description': 'TODO' },
            'provider': {
               'Required': 1,
               'Typecast': GTypecast.name,
               'Description': 'TODO' },
            'dbname': {
               'Required': 0,
               'Typecast': GTypecast.text,
               'Description': 'TODO' },
            'service': {
               'Required': 0,
               'Typecast': GTypecast.text,
               'Description': 'TODO' },
            'comment': {
               'Required': 0,
               'Typecast': GTypecast.text,
               'Description': 'TODO' },
            'host': {
               'Required': 0,
               'Typecast': GTypecast.text,
               'Description': 'TODO' } },
         'ParentTags': ('form',),
         'Description': 'TODO' },

      'page': {
         'BaseClass': GFObjects.GFPage,
         'Required': 1,
         'Importable':1,
         'Attributes': {
            'name': {
               'Unique': 1,
               'Typecast': GTypecast.name,
               'Description': 'A unique ID for the widget. This is only useful when importing pages from a library.' },
            'caption': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' } },
         'ParentTags': ('form',),
         'Description': 'TODO' },

      'block': {
         'BaseClass': GFObjects.GFBlock,
         'Required': 1,
         'Importable':1,
         'Attributes': {
            'name': {
               'Required': 1,
               'Unique': 1,
               'Typecast': GTypecast.name,
               'Description': 'A unique ID for the widget.The name of the widget. No blocks or datasources can share the same name without causing namespace collisions in user triggers.' },
            'rows': {
               'Typecast': GTypecast.whole,
               'Description': 'Any widgets inside the block will display this number of copies in a verticle column. Simulates a very crude grid entry system. Serves the same purpose as the visibleCount attribute on some widgets.'},
            'rowSpacer': {
               'Typecast': GTypecast.whole,
               'Description': 'Adjusts the vertical gap of this number of rows between duplicated widgets. Serves the same purpose as some of the gap attributes on individual widgets.' },
            'transparentBlock':{
               'Typecast': GTypecast.boolean,
               'Default': 0,
               'Description': 'If set then the you can tab out of the block via next or previous field events. Makes navigation in mutliblock forms easier. ' },
            'restrictDelete':{
               'Typecast': GTypecast.boolean,
               'Default': 0,
               'Description': 'If set then the user will be unable to request that a record be deleted via the user interface.' },
            'restrictInsert':{
               'Typecast': GTypecast.boolean,
               'Default': 0,
               'Description': 'If set then the user will be unable to request that new records be inserted into the block' },
            'datasource': {
               'References': (('datasource','name'),),
               'Typecast': GTypecast.name,
               'Description': 'The name of a datasource (defined in by a '
                              '{<datasource>} tag.) that provides this block '
                              'with it\'s data.' } },
         'ParentTags': ('page',),
         'Description': 'TODO' },

      'label': {
         'BaseClass': GFObjects.GFLabel,
         'Attributes': {
            'name': {
               'Unique': 1,
               'Typecast': GTypecast.name,
               'Description': 'The unique ID of the label.' },
            'text': {
               'Required': 1,
               'Typecast': GTypecast.text,
               'Description': 'The text to be displayed.' },
            'alignment': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                  'left': {},
                  'right': {},
                  'center': {} },
               'Default': "left",
               'Description': 'The justification of the label. Can be one of '
                              'the following: {left}, {right}, or {center}. '
                              'Requires that the {width} attribute be set.'},
            'width': {
               'Typecast': GTypecast.whole,
               'Description': 'The width of the label in text columns. '
                              'Defaults to the width of the text. Only really '
                              'useful when used with the {alignment} attribute.' },
            'rows': {
               'Typecast': GTypecast.whole,
               'Description': 'Overrides the rows setting defined at the block level. ' },
            'rowSpacer': {
               'Typecast': GTypecast.whole,
               'Description': 'Overriders the rowSpace setting defined at the block level.' },
            'x': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The text column starting position of the widget. Based upon leftmost column of screen being 0. ' },
            'y': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The text row starting position of the widget. Based upon the top row of the screen being 0.' } },
         'ParentTags': ('page','block',),
         'Description': 'TODO' },

      # If you implement a new entry "style", add to the entryStyles
      # structure after this list
      'entry': {
         'BaseClass': GFObjects.GFEntry,
         'Importable':1,
         'Attributes': {
            'name': {
               'Required': 1,
               'Unique': 1,
               'Typecast': GTypecast.name,
               'Description': 'The unique ID of the entry. Referenced in '
                              'master/detail setups as well as triggers.' },
            'field': {
               'Typecast': GTypecast.name,
               'Description': 'The name of the field in the datasource to '
                              'which this widget is tied.' },
            'height': {
               'Typecast': GTypecast.whole,
               'Default': 1,
               'Description': 'The height of the entry in text rows. ' },
            'width': {
               'Typecast': GTypecast.whole,
               'Description': 'The width of the entry in text columns.' },
            'max_length': {
               'Typecast': GTypecast.whole,
               'Description': 'The maximum number of characters the user is '
                              'allowed to enter into the entry.' },
            'visibleCount':{
               'Typecast': GTypecast.whole,
               'Deprecated': 'Use the <block> "rows" attribute instead.' },
            'focusorder': {
               'Typecast': GTypecast.whole,
               'Description': 'TODO'},
            'rows': {
               'Typecast': GTypecast.whole,
               'Description': 'TODO'},
            'rowSpacer': {
               'Typecast': GTypecast.whole,
               'Description': 'TODO' },
            'readonly': {
               'Typecast': GTypecast.boolean,
               'Description': 'It defined the user will be unable to alter '
                              'the contents of this entry. Triggers can still '
                              'alter the value.',
               'Default': 0   },
            'required': {
               'Description': 'This object cannot have an empty value prior '
                              'to a commit.',
               'Typecast': GTypecast.boolean,
               'Default': 0   },
            'uppercase': {
               'Deprecated': 'Use case="upper" instead.',
               'Typecast': GTypecast.boolean,
               'Default': 0  },
            'lowercase': {
               'Deprecated': 'Use case="lower" instead.',
               'Typecast': GTypecast.boolean,
               'Default': 0   },
            'numeric': {
               'Deprecated': 'Use typecast="number" instead',
               'Typecast': GTypecast.boolean,
               'Default': 0   },
            'hidden': {
               'Typecast': GTypecast.boolean,
               'Default': 0,
               'Description': 'If defined the entry widget will not be '
                              'displayed on the form. This is usefull for '
                              'fields the user doesn\'t need to know about '
                              'that you wish to update via triggers.'   },
            'style': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                  'default': {},
                  'dropdown': {},
                  'checkbox': {},
                  'label': {} },
               'Default': 'default',
               'Description': 'The style of entry widget requested. Currently '
                              'either {text}, {label}, {checkbox}, or {dropdown}. To '
                              'use {dropdown} you are required to use both the '
                              '{fk_source}, {fk_key}, and {fk_description} '
                              'attributes. The {label} style implies the '
                              '{readonly} attribute.'  },
            'case': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                  'mixed': {},
                  'upper': {},
                  'lower': {} },
               'Default': 'mixed',
               'Description': 'TODO'  },
            'typecast': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                  'text': {},
                  'number': {},
                  'date': {} },
               'Default': 'text',
               'Description': 'The type of data the entry widget will accept. '
                              'Possible values are {text}, {number}, {date}.'},
            'formatmask': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' },
            'inputmask': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' },
            'displaymask': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' },
            'value': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' },
            'foreign_key': {
               'Typecast': GTypecast.name,
               'Deprecated': 'Use fk_source="..." and fk_key="..." instead.',
               'Description': 'Set in the format datasource.field This forces any '
                              'input into this entry to match the values stored '
                              'in the datasource.field' },
            'foreign_key_description': {
               'Typecast': GTypecast.text,
               'Deprecated': 'Use fk_description="..." instead.',
               'Description': 'The description used if a style of dropdown is '
                              'selected. This fields value is displayed in the '
                              'dropdown but the foreign_key value is actually '
                              'stored in the field. This allows you to display '
                              'something like the full name of a US state but '
                              'only store it\'s 2 character abrievation.' },
            'fk_source': {
               'Typecast': GTypecast.name,
               'Description': 'TODO' },
            'fk_key': {
               'Typecast': GTypecast.name,
               'Description': 'TODO' },
            'fk_description': {
               'Typecast': GTypecast.name,
               'Description': 'TODO' },
            'default': {
               'Typecast': GTypecast.text,
               'Description': 'The default value for any new records created. '
                              'If the field is visible the user can override '
                              'the value.' },
            'queryDefault':{
               'Typecast': GTypecast.text,
               'Description': 'The form will be populated with this value '
                              'automatically when a query is requested. If '
                              'the field is visible the user can still '
                              'override the value.' },
            'sloppyQuery': {
               'Typecast': GTypecast.text,
               'Description': 'When set, whatever value the user enters for '
                              'the query mask is rewritten with % between '
                              'each character. Thus {example} would be '
                              'queried as {%e%x%a%m%p%l%e%}' },
            'ignoreCaseOnQuery': {
               'Typecast': GTypecast.boolean,
               'Default': 0,
               'Description': 'If defined the entry widget ignores the case '
                              'of the information entered into the query mask.' },
            'editOnNull': {
               'Description': 'Only allow this object to be edited if it '
                              'is currently empty.',
               'Typecast': GTypecast.boolean,
               'Default': 0,
               'Description': 'TODO' },
            'no_ltrim': {
               'Description': 'Suppress trimming of extraneous space at '
                              'beginning of user input.',
               'Typecast': GTypecast.boolean,
               'Default': 0,
               'Description': 'TODO' },
            'no_rtrim': {
               'Description': 'Suppress trimming of extraneous space at end '
                              'of user input.',
               'Typecast': GTypecast.boolean,
               'Default': 0,
               'Description': 'TODO' },
            'x': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The text column starting position of the '
                              'widget. Based upon leftmost column of screen '
                              'being 0.' },
            'y': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The text row starting position of the widget. '
                              'Based upon the top row of the screen being 0.' } },
         'ParentTags': ('block','box'),
         'Description': 'TODO' },

      'scrollbar': {
         'BaseClass': GFObjects.GFScrollBar,
         'Attributes': {
            'width': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The width of the box in text columns.' },
            'height': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The height of the box in text rows.' },
            'x': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The text column starting position of the widget. Based upon leftmost column of screen being 0. ' },
            'y': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The text row starting position of the widget. Based upon the top row of the screen being 0.' } },
         'ParentTags': ('page','block','box'),
         'Description': 'TODO' },

      'box': {
         'BaseClass': GFObjects.GFBox,
         'Attributes': {
            'name': {
               'Unique': 1,
               'Typecast': GTypecast.name,
               'Description': 'TODO' },
            'label': {
               'Typecast': GTypecast.text,
               'Description': 'An optional text label that will be displayed on the border.' },
            'width': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The width of the box in text columns.' },
            'height': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The height of the box in text rows.' },
            'focusorder': {
               'Typecast': GTypecast.whole,
               'Description': 'TODO'  },
            'x': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The text column starting position of the widget. Based upon leftmost column of screen being 0. ' },
            'y': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The text row starting position of the widget. Based upon the top row of the screen being 0.' } },
         'ParentTags': ('page','block',),
         'Description': 'TODO' },

      'button': {
         'BaseClass': GFObjects.GFButton,
         'Importable':1,
         'Attributes': {
            'name': {
               'Unique': 1,
               'Typecast': GTypecast.name,
               'Description': 'A unique ID for the widget. Useful for importable buttons. ' },
            'trigger': {
               'Typecast': GTypecast.name,
               'Description': 'The name of a named trigger that this button will fire when pressed.' },
            'focusorder': {
               'Typecast': GTypecast.whole,
               'Description': 'TODO'},
            'label': {
               'Typecast': GTypecast.name,
               'Description': 'The text that should appear on the button' },
            'width': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The width of the entry in text columns.' },
            'height': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The height of the entry in text rows. ' },
            'x': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The text column starting position of the widget. Based upon leftmost column of screen being 0.' },
            'y': {
               'Required': 1,
               'Typecast': GTypecast.whole,
               'Description': 'The text row starting position of the widget. Based upon the top row of the screen being 0.' } },
         'ParentTags': ('page','block','box'),
         'Description': 'TODO' },

      'options': {
         'BaseClass': GFObjects.GFOptions,
         'UsableBySiblings': 1,
         'ParentTags': ('form',),
         'Description': 'TODO' },

      'option': {
         'BaseClass': GFObjects.GFOption,
         'Attributes': {
            'name': {
               'Required': 1,
               'Typecast': GTypecast.name,
               'Description': 'TODO' },
            'value': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' } },
         'MixedContent': 1,
         'ParentTags': ('options',),
         'Description': 'TODO' },

      'title': {
         'BaseClass': GFObjects.GFOption,
         'Attributes': {
            'name': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                   'title': {} },
               'Default': 'title',
               'Description': 'TODO' },
            'value': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' } },
         'MixedContent': 1,
         'SingleInstance': 1,
         'Deprecated': 'Use the <form> attribute "title" instead.',
         'ParentTags': ('options',),
         'Description': 'TODO' },

      'name': {
         'BaseClass': GFObjects.GFOption,
         'Attributes': {
            'name': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                   'name': {} },
               'Default': 'name',
               'Description': 'TODO' },
            'value': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' } },
         'MixedContent': 1,
         'SingleInstance': 1,
         'ParentTags': ('options',),
         'Description': 'TODO' },

      'height': {
         'BaseClass': GFObjects.GFOption,
         'Attributes': {
            'name': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                   'height': {} },
               'Default': 'height',
               'Description': 'TODO' },
            'value': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' } },
         'MixedContent': 1,
         'SingleInstance': 1,
         'Deprecated': 'Use the <form> attribute "height" instead.',
         'ParentTags': None,
         'Description': 'TODO' },

      'width': {
         'BaseClass': GFObjects.GFOption,
         'Attributes': {
            'name': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                   'width': {} },
               'Default': 'width' },
            'value': {
               'Typecast': GTypecast.text } },
         'MixedContent': 1,
         'SingleInstance': 1,
         'Deprecated': 'Use the <form> attribute "width" instead.',
         'ParentTags': ('options',),
         'Description': 'TODO' },

      'author': {
         'BaseClass': GFObjects.GFOption,
         'Attributes': {
            'name': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                   'author': {} },
               'Default': 'author',
               'Description': 'TODO' },
            'value': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' } },
         'MixedContent': 1,
         'SingleInstance': 1,
         'ParentTags': ('options',),
         'Description': 'TODO' },

      'description':{
         'BaseClass': GFObjects.GFOption,
         'Attributes': {
            'name': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                   'description': {} },
               'Default': 'description',
               'Description': 'TODO' },
            'value': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' } },
         'MixedContent': 1,
         'SingleInstance': 1,
         'ParentTags': ('options',),
         'Description': 'TODO' },

      'version': {
         'BaseClass': GFObjects.GFOption,
         'Attributes': {
            'name': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                   'version': {} },
               'Default': 'version',
               'Description': 'TODO' },
            'value': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' } },
         'MixedContent': 1,
         'SingleInstance': 1,
         'ParentTags': ('options',),
         'Description': 'TODO' },

      'tip': {
         'BaseClass': GFObjects.GFOption,
         'Attributes': {
            'name': {
               'Typecast': GTypecast.name,
               'ValueSet': {
                   'tip': {} },
               'Default': 'tip',
               'Description': 'TODO' },
            'value': {
               'Typecast': GTypecast.text,
               'Description': 'TODO' } },
         'MixedContent': 1,
         'SingleInstance': 1,
         'ParentTags': ('options',),
         'Description': 'TODO' },
      'parameter':    {
         'BaseClass': GFObjects.GFParameter,
         'Attributes': {
            'name':          {
               'Required': 1,
               'Unique': 1,
               'Typecast': GTypecast.name,
               'Description': 'TODO' },
            'required':    {
               'Typecast': GTypecast.boolean,
               'Default': 0,
               'Description': 'TODO' },
##            'limited':     {
##               'Typecast': GTypecast.boolean,
##               'Default': 0,
##               'Description': 'TODO' },
            'default':     {
               'Typecast': GTypecast.text,
               'Description': 'TODO' },
            'description': {
##               'Required': 1,
               'Typecast': GTypecast.text,
               'Description': 'TODO' },
##            'source':      {
##               'Typecast': GTypecast.name,
##               'Description': 'TODO' },
            'type':        {
               'Typecast': GTypecast.name,
               'Default': "char",
               'Description': 'TODO' } },
         'ParentTags':  ('form',),
         'Description': 'TODO' },
    }


    #
    # Add DataSource elements
    #
    xmlElements.update(
      GDataSource.getXMLelements(
        updates={'datasource': {
                   'BaseClass': GFObjects.GFDataSource,
                   'ParentTags': ('form',) },
                 'cparam': {
                   'BaseClass': GFObjects.GFCParam }
                 }))


    #
    # Add trigger elements
    #
    xmlElements.update(
      GTrigger.getXMLelements(
        updates={'trigger':{
                   'ParentTags': ('form',)},
                }))

    #
    # Add menu elements
    #
    xmlElements.update(
        GMenu.getXMLelements(
                updates={'menu': {
                   ##'BaseClass': GFObjects.GFMenu,
                   'ParentTags': ('form',) },
                 }))

  return GParser.buildImportableTags('form',xmlElements)

#
# Different Types of Entries
#
entryStyles = {'text': 'Text Field',
               'dropdown': 'Drop Down Box',
               'checkbox': 'Check Box' }

#######################################################
#
# xmlFormsHandler
#
# This class is called by the XML parser to
# process the xml file.
#
#######################################################

class xmlFormsHandler (GParser.xmlHandler):
  def __init__(self):

    GParser.xmlHandler.__init__(self)

    self.xmlElements = getXMLelements()


