#
# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2002 Free Software Foundation
#
# FILE:
# UIwxpython.py
#
# DESCRIPTION:
# A wxPython based user interface driver for GNUe forms.
#
# NOTES:
#  TODO: why does UIhelper not inherit from UIwidget?
#

from wxPython.wx import *
from wxPython.lib.splashscreen import SplashScreen
from gnue.forms.GFForm import *
from gnue.common import events
from gnue.forms import GFKeyMapper
from gnue.forms.uidrivers._base.UIdriver import *
from gnue.forms.uidrivers._base.UIWidget import *

import os
import sys
import string
from gnue.forms import VERSION
from gnue.common import GDebug, GLoginHandler
from gnue.common import GConfig
from gnue.common.TextUtils import lineWrap

_LOOPTRAP = 0
_PROMPTFORRECORD = None
_charWidth = 0
_charHeight = 0
_IdToWxObj = {}
_IdToGFObj = {}
_IdToUIObj = {}

_disabledColour = wxLIGHT_GREY

# Finishing creation of dictionary for language font encodings

encodings = {
	'iso8859-1': wxFONTENCODING_ISO8859_1,
	'iso8859-2': wxFONTENCODING_ISO8859_2,
	'iso8859-3': wxFONTENCODING_ISO8859_3,
	'iso8859-4': wxFONTENCODING_ISO8859_4,
	'iso8859-5': wxFONTENCODING_ISO8859_5,
	'iso8859-6': wxFONTENCODING_ISO8859_6,
	'iso8859-7': wxFONTENCODING_ISO8859_7,
	'iso8859-8': wxFONTENCODING_ISO8859_8,
	'iso8859-9': wxFONTENCODING_ISO8859_9,
	'iso8859-10': wxFONTENCODING_ISO8859_10,
	'iso8859-11': wxFONTENCODING_ISO8859_11,
	'iso8859-12': wxFONTENCODING_ISO8859_12,
	'iso8859-13': wxFONTENCODING_ISO8859_13,
	'iso8859-14': wxFONTENCODING_ISO8859_14,
	'iso8859-15': wxFONTENCODING_ISO8859_15,
	'koi8': wxFONTENCODING_KOI8,
	'koi8-r': wxFONTENCODING_KOI8,
	'cp1250': wxFONTENCODING_CP1250,
	'cp1251': wxFONTENCODING_CP1251,
	'cp1252': wxFONTENCODING_CP1252
}

# hack for wxPython 2.2 (2.3+ doesn't need this)
def _eventObjTowxWindow(event):
  eo = event.GetEventObject()
  return wxPyTypeCast(eo, 'wxWindow')

##
## Only create one instance of wxApp
## Use getWxApp() to get or create the
## single instance
##
## TODO: It seems this could all be folded
## TODO: into an __init__ inside GFwxApp
##
__wxApp = None

def getWxApp():
  global __wxApp

  if not __wxApp:
    GDebug.printMesg(7,"WxApp initializing")
    __wxApp = GFwxApp(0)
  return __wxApp


class GFwxApp(wxApp):
  _mainLoop= None

  def OnInit(self):
    GDebug.printMesg(7,"WxApp initializing")
    wxInitAllImageHandlers()
    return true

  def MainLoop (self):
    # just to launch one wx's mainloop
    # otherwise, the app does not finishes cleanly when several windows are open
    # 'cause the first mainloop gets all the events
    # and the others just sit and wait for something to happen
    if self._mainLoop==None:
      self._mainLoop= 1
      wxApp.MainLoop (self)

      
#
# Little global helper routine to set font according to options
#
# TODO: Not completed
#
def initFont(widget, affectsLayout=1):
  if int(gConfigForms('fixedWidthFont')):
    widget.SetFont(wxFont(int(gConfigForms('pointSize')),wxMODERN,wxNORMAL,\
      wxNORMAL,FALSE,'',encodings[gConfigForms('formFontEncoding')]))

#
# GFUserInterface
#
# The public interface to the User Interface
# All UIs must provide this class
#
class GFUserInterface(GFUserInterfaceBase):
  def __init__(self, eventController, disableSplash = None):

    self._WIDGETS = WIDGETS

    self._DIALOGS = {'about'       :UIAbout,
                     'messageBox'  :UIMessageBox,
                     }

    self._aboutDialog = UIAbout
    self._messageDialog = UIMessageBox

    self._wxapp = getWxApp()
    self._disableSplash = disableSplash
    GFUserInterfaceBase.__init__(self,eventController)

    global _PROMPTFORRECORD
    _PROMPTFORRECORD = self.promptForRecordNumber

    self.forms = {}
    self.init()

  #
  # init
  #
  # Routine called by wxWindows when wxApp instance is created
  # It is used the build the basic UI
  #
  def init(self):

    #
    # SplashScreen
    #
    if not self._disableSplash:
      self.splash = SplashScreen(None,bitmapfile=images_dir+gConfigForms('splashScreenPNG'),
                                 duration = 3000, callback=self.onCloseSplash,
                                 style=wxSIMPLE_BORDER|wxCENTRE_ON_SCREEN )
      swidth,sheight = self.splash.GetSizeTuple()

      # Make the splash text print white
      self.splash.SetForegroundColour(wxColour(0xFF, 0xFF, 0xFF))
      self.splash.SetFont(wxFont(20,wxSWISS,wxNORMAL,wxBOLD))

      text = _("GNU Enterprise Forms")
      twidth,theight = self.splash.GetTextExtent(text)
      wxStaticText(parent=self.splash, id=-1, label=str(text),
                   size=wxSize(swidth,theight),pos=wxPoint(1,1),style=wxALIGN_CENTRE)

      self.splash.SetFont(wxFont(10,wxSWISS,wxNORMAL,wxBOLD))

      text = _("Version %s") % VERSION
      twidth,theight = self.splash.GetTextExtent(text)
      wxStaticText(parent=self.splash, id=-1, label=str(text),
                   size=wxSize(swidth,theight), pos=wxPoint(1,25),style=wxALIGN_CENTRE)

      text = "(c)2000-2002 Free Software Foundation"
      twidth,theight = self.splash.GetTextExtent(text)
      wxStaticText(self.splash, -1, str(text),
                   wxPoint(1,50),wxSize(swidth,theight),wxALIGN_CENTRE)

      text = _("Protected by GNU General Public Licence 2.0")
      wxStaticText(self.splash, -1, str(text),
                   wxPoint(1,sheight-(theight+10)),wxSize(swidth-5,theight),wxALIGN_RIGHT)

      self.splash.CenterOnScreen()
      self.splash.Show(true)

    #
    # Create the main window
    #
    self.mainWindow = wxFrame(NULL, -1, "", wxDefaultPosition)
    initFont(self.mainWindow,1)

    #
    # Find the max size of printable characters
    #  used to setup the grid
    #
    maxWidth, maxHeight, maxDescent, maxLeading = [0,0,0,0]
    checkchars = string.printable
    for letter in checkchars:
      width,height,descent,leading = self.mainWindow.GetFullTextExtent(letter)
      maxWidth = max(maxWidth,width)
      maxHeight = max(maxHeight,height)
      maxDescent = max(maxDescent,descent)
      maxLeading = max(maxLeading,leading)

    self.charWidth = int(maxWidth+maxLeading)
    self.charHeight = int(maxHeight+maxDescent)

    self.widgetWidth = self.charWidth       # The pixel width of a 1 char widget (for things like buttons)
    self.widgetHeight = self.charHeight + 3 # The pixel height of a 1 char widget (for things like buttons)
    self.textWidth = self.charWidth         # The pixel width of text inside a widget
    self.textHeight = self.charHeight       # The pixel height of text inside a widget

    global _charWidth, _charHeight
    _charWidth = self.charWidth             # Used by the mouse handler to compute char offset into widget
    _charHeight = self.charHeight           #


    self.menu_sb_space=4 # the extra spaces needed by the menu, toolbar and statusbar

    self.statusBar = self.mainWindow.CreateStatusBar()
    self.statusBar.SetFieldsCount(5)
    self.statusBar.SetStatusWidths([-1,50,50,75,75])
    initFont(self.statusBar,0)


    #self.mainWindow.panel = wxPanel(self.mainWindow,-1, wxDefaultPosition,self.mainWindow.GetClientSize())
    self.mainWindow.panel = wxScrolledWindow(self.mainWindow,-1, wxDefaultPosition,self.mainWindow.GetClientSize())


    global _disabledColour
    _disabledColour = self.mainWindow.GetBackgroundColour()

    self.currentWidget = [self.mainWindow.panel]
    #self._wxapp.SetTopWindow(self.mainWindow)

    EVT_CLOSE(self.mainWindow,self.closeTrap)

    return true


  def activateForm(self, form):

    self.menu_sb_space = 0
    if not form._features['GUI:MENUBAR:SUPPRESS']:
      self.menu_sb_space += 2


      # This will be a dict of the form { 'requestROLLBACK': 1001 }
      # where 1001 is the wxId associated with requestROLLBACK.
      self.__eventMenuMapping = {}


      fileMenu = wxMenu()
      self.__addMenuItem(fileMenu, _("Save       (F6)"),_('Save all changes to database.'),'requestCOMMIT')
      self.__addMenuItem(fileMenu, _("Clear Form (F11)"),_('Clear Form'),'requestROLLBACK')
      fileMenu.AppendSeparator()

      self.__addMenuItem(fileMenu, _("&Print"),_('Perform print routine for this form.'),'requestPRINTOUT')
      fileMenu.AppendSeparator()

      id = self.__addMenuItem(fileMenu, _("E&xit"),_('Leave the application.'))
      EVT_MENU(self._wxapp, id,  self.menuExitEvent)
      self.__eventMenuMapping['_exit_'] = id

      editMenu = wxMenu()
      self.__addMenuItem(editMenu, _("Cut"),_("Cut the selected text"),'requestCUT')
      self.__addMenuItem(editMenu, _("Copy"),_("Copy the selected text"), 'requestCOPY')
      self.__addMenuItem(editMenu, _("Paste"),_("Paste text"),'requestPASTE')

      dataMenu = wxMenu()
      self.__addMenuItem(dataMenu, _("First Record        (Ctrl+Up)"),_('Navigate to first record in memory.'),'requestFIRSTRECORD')
      self.__addMenuItem(dataMenu, _("Previous Record         (Up)"),_('Navigate to previous record in memory.'),'requestPREVRECORD')
      self.__addMenuItem(dataMenu, _("Next Record           (Down)"),_('Navigate to next record in memory.'),'requestNEXTRECORD')
      self.__addMenuItem(dataMenu, _("Last Record    (Ctrl+Down)"),_('Navigate to last record in memory.'),'requestLASTRECORD')
      id = self.__addMenuItem(dataMenu, _("Jump to Record...        (F2)"),_('Prompts for a record number to which the system should jump.'))
      EVT_MENU(self._wxapp, id, _PROMPTFORRECORD)
      self.__eventMenuMapping['_promptForRecord_'] = id

      self.__addMenuItem(dataMenu, _("New Record              (F12)"),_('Create a new record for data input.'),'requestNEWRECORD')
      self.__addMenuItem(dataMenu, _("Mark Record for Delete   (F5)"),_('Mark record for removal at next commit.'),'requestMARKFORDELETE')

      dataMenu.AppendSeparator()
      self.__addMenuItem(dataMenu, _("Next Block              (PgDn)"),_('Navigate to next data block.'),'requestNEXTBLOCK')
      self.__addMenuItem(dataMenu, _("Previous Block       (PgUp)"),_('Navigate to previous data block.'),'requestPREVBLOCK')

      dataMenu.AppendSeparator()
      self.__addMenuItem(dataMenu, _("Enter Query             (F8)"),_('Switch to input query mask mode.'),'requestENTERQUERY')
      self.__addMenuItem(dataMenu, _("Execute Query           (F9)"), _('Execute query using current mask.'),'requestEXECQUERY')

      helpMenu = wxMenu()
      self.__addMenuItem(helpMenu, _("&About..."), _('Display info about GNUe Forms.'),'requestABOUT')

      menuBar = wxMenuBar()
      menuBar.Append( fileMenu, _("&File") )
      menuBar.Append( editMenu, _("&Edit") )
      menuBar.Append( dataMenu, _("&Data") )
      menuBar.Append( helpMenu, _("&Help") )

      self._menuBar = menuBar
      self.mainWindow.SetMenuBar( self._menuBar );


    if not form._features['GUI:TOOLBAR:SUPPRESS']:

      self.menu_sb_space += 2
      #
      #  There has got to be better way to call events to ride off the EVT_MENU
      #  Ok if i give buttons same id as menu's they share events
      #  I dont know if this is horribly wrong or design
      #
      if sys.platform == 'win32':
        mainToolBar = wxToolBar(self.mainWindow, -1)
      else:
        mainToolBar = wxToolBar(self.mainWindow,-1, style=wxTB_HORIZONTAL|wxTB_DOCKABLE)

      mainToolBar.SetToolBitmapSize(wxSize(24,24))
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['requestCOMMIT'], wxImage(images_dir+gConfigForms('tb_commit'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Save  Changes"), _('Save all changes to database.'))
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['requestNEWRECORD'], wxImage(images_dir+gConfigForms('tb_insert'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Insert Record"), _('Create a new record for data input.'))
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['requestMARKFORDELETE'], wxImage(images_dir+gConfigForms('tb_delete'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Delete Record"), _('Mark record for removal at next commit.'))
      mainToolBar.AddSeparator()
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['requestFIRSTRECORD'], wxImage(images_dir+gConfigForms('tb_first'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("First Record"),_('Navigate to first record in memory.'))
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['requestPREVRECORD'], wxImage(images_dir+gConfigForms('tb_previous'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Previous Record"),_('Navigate to previous record in memory.'))
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['requestNEXTRECORD'], wxImage(images_dir+gConfigForms('tb_next'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Next Record"),_('Navigate to next record in memory.'))
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['requestLASTRECORD'], wxImage(images_dir+gConfigForms('tb_last'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Last Record"),_('Navigate to last record in memory.'))
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['_promptForRecord_'], wxImage(images_dir+gConfigForms('tb_jump'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Jump to Record"),_('Navigate to user specified record in memory.'))
      mainToolBar.AddSeparator()
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['requestENTERQUERY'], wxImage(images_dir+gConfigForms('tb_query_prep'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Prepare Query"), _('Switch to input query mask mode.'))
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['requestEXECQUERY'], wxImage(images_dir+gConfigForms('tb_query'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Execute Query"), _('Execute query using current mask.'))
      mainToolBar.AddSeparator()
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['requestROLLBACK'], wxImage(images_dir+gConfigForms('tb_rollback'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Clear Form"), _('Clear Form'))
      mainToolBar.AddSimpleTool(self.__eventMenuMapping['_exit_'], wxImage(images_dir+gConfigForms('tb_exit'), wxBITMAP_TYPE_PNG).ConvertToBitmap(), _("Exit Form"), _('Exit Form'))

      self._mainToolBar = mainToolBar
      self.mainWindow.SetToolBar( mainToolBar );
      self._mainToolBar.Realize()


    width = int(self._form.width)
    height = int(self._form.height)

    formSize = wxSize(width*self.widgetWidth,
                      (height+self.menu_sb_space)*self.widgetHeight)

    self.mainWindow.SetSize(formSize)
    self.mainWindow.SetTitle(str(self._form.title))

    if self._notebook: # Adjust sizes
      self._notebook.SetClientSize(formSize)

      child = self._notebook.GetParent()
      while child:
        child.Fit()
        child = child.GetParent()

    # This is the initial panel window setup during init
    # not a wxPanel set per page right after this
    tempx,tempy =  self.mainWindow.GetClientSizeTuple()
    self.mainWindow.panel.SetScrollbars(1, 1, tempx, tempy)


    self.mainWindow.panel =  self._pageList[0]
    self.mainWindow.panel.SetSize(self.mainWindow.GetClientSize())
    self._pageList[0].Show(TRUE)


    # Only one page at a time can be visible
    self.visiblePage = self._pageList[0]

    self._wxapp.SetTopWindow(self.mainWindow)

    tempx,tempy = self.mainWindow.GetSizeTuple()
    mainWindowSize = wxSize(20+tempx, 20+tempy)

    self.mainWindow.SetSize(mainWindowSize)

    self.mainWindow.CenterOnScreen()
    self.mainWindow.Show(true)
    # Keep splash on top
    if hasattr(self,'splash'):
      self.splash.Raise()


  #############################################################################
  #
  # Private UIBase support functions
  #
  # Called in UIbase functions to perform UI interaction with this specific
  # widget set.
  #

  #
  # _exit
  #
  # Tells the application to close it's main window
  #
  def _exit(self):
    self.mainWindow.Close(true)


  #
  # Internal helper function to add a menu item and an event
  #
  def __addMenuItem(self, menu, text, help, event=None):
    id = wxNewId()
    menu.Append(id, text, help)
    if event:
      EVT_MENU(self._wxapp, id,
        lambda event, l=self, e=event: l.dispatchEvent(events.Event(e)))
      self.__eventMenuMapping[event] = id
    return id


  #
  # _setStatusBar
  #
  def _setStatusBar(self,tip, statusValue, insertValue, currentRecord, maxRecord, currentPage, maxPage):
    if tip: self.statusBar.SetStatusText(str(tip),0)

    if statusValue:
      self.statusBar.SetStatusText(statusValue,1)

    if insertValue:
      self.statusBar.SetStatusText(insertValue,2)

    if currentRecord and  maxRecord:
      self.statusBar.SetStatusText(string.strip("%s/%s" % (currentRecord,maxRecord)),3)

    if currentPage and  maxPage:
      self.statusBar.SetStatusText(string.strip("%s/%s" % (currentPage,maxPage)),4)

    #
    # Adjust the status fields to reflect width of text in them
    #
    # This is a hack
    #
    tipWidth = -1 # Fill whatever remains
    statusWidth,unused = self.statusBar.GetTextExtent(self.statusBar.GetStatusText(1))
    insertWidth,unused = self.statusBar.GetTextExtent(self.statusBar.GetStatusText(2))
    recordWidth,unused = self.statusBar.GetTextExtent(self.statusBar.GetStatusText(3))
    pageWidth,unused = self.statusBar.GetTextExtent(self.statusBar.GetStatusText(4))

    # Either double the width if it's set or default to 5
    statusWidth = statusWidth and (statusWidth * 2) or 5
    insertWidth = insertWidth and (insertWidth * 2) or 5
    recordWidth = recordWidth and (recordWidth * 2) or 5
    pageWidth   = pageWidth   and (pageWidth   * 2) or 5

    self.statusBar.SetStatusWidths([tipWidth,statusWidth,insertWidth,recordWidth,pageWidth])


  # Can be used by UI drivers to add more attributes to the creation event.
  def _updateCreateEvent(self, event):
    event.eventHandler = self.dispatchEvent
    event.ui = self
    return event


  #############################################################################
  #
  # Incoming Event Processors
  #
  # Processes the incoming events from other objects
  #

  #
  # mainLoop
  #
  # The primary loop of the user interface.  Called once the UI is
  # fully activated
  #
  def mainLoop(self):
    self._wxapp.MainLoop() # simply call the wxApp's MainLoop method

  #
  # gotoPage
  #
  # makes the requested page visible on the screen
  #
  def gotoPage(self, event):
    if not self._notebook:
      self.visiblePage.Show(FALSE)
      self.visiblePage = self._formToUI[event.data][0]
      # TODO: move the calculatoin out of here
      self.visiblePage.SetSize(wxSize(self._form.width*self.widgetWidth,
                                      (self._form.height+self.menu_sb_space)*self.widgetHeight))
    else:
      self.visiblePage = self._formToUI[event.data][0]
      for count in range(self._notebook.GetPageCount()):
        if self.visiblePage.GetId() == self._notebook.GetPage(count).GetId():
          self._notebook.SetSelection(count)
          break
    self.visiblePage.Show(TRUE)

  #
  # formAlert
  #
  # Rings a bell and alters the statusbar to display
  # a line of text
  #
  def formAlert(self, event):
    wxBell()
    self.statusBar.SetStatusText(event.data,0)

  #
  # Called whenever forms goes into a "wait" state in which user cannot
  # interact with interface (e.g., while waiting for a query or a commit)
  #
  def beginWait (self, event):
    wxBeginBusyCursor()

  #
  # Called whenever forms leaves a "wait" state
  #
  def endWait (self, event):
    wxEndBusyCursor()


  #
  # Clipboard routines
  #
  # If a particular UI has a system-wide clipboard,
  # these methods should be overridden to use that
  # clipboard.
  #
  def getClipboardContents(self, event):

    if wxTheClipboard.Open():
      data = wxTextDataObject()
      success = wxTheClipboard.GetData(data)
      wxTheClipboard.Close()
    else:
      success = 0
      GDebug.printMesg(5,'Unable to open clipboard for read')

    if success:
      value = data.GetText()
    else:
      GDebug.printMesg(5,'Unable to obtain clipboard contents. Defaulting to Empty.')
      value = None

    GDebug.printMesg(5, "Getting clipboard value '%s'" % value)
    event.__result__ = value


  def setClipboardContents(self, event):

    GDebug.printMesg(5,"Setting clipboard '%s'" % event.text)

    if wxTheClipboard.Open():
      value = wxTheClipboard.SetData(wxTextDataObject(event.text))
      GDebug.printMesg(10,"Set Clipboard Status: %s" % value)
      wxTheClipboard.Close()

    else:
      GDebug.printMesg(5,'Unable to open clipboard for write')


  #############################################################################
  #
  # Internal Event Processors
  #
  # Processes the events from the widget set
  #

  #
  # menuExitEvent
  #
  # Catches the Exit event from the menu and make sure it closes in such a way to
  # trigger the closeTrap method
  #
  def menuExitEvent(self, event):
    self.mainWindow.Close()

  #
  # promptForRecordNumber
  #
  # Presents a dialog to user asking for the record number to jump to
  #
  def promptForRecordNumber(self, event= None):
    dlg = wxTextEntryDialog(self.mainWindow, _('Enter record number to jump to'),
                                                        _('Which record?'))
    if dlg.ShowModal() == wxID_OK:
      action = self.dispatchEvent(events.Event('requestRECORDNUMBER', dlg.GetValue()))
    else:
      action = None

    dlg.Destroy()
    return action

  #
  # closeTrap
  #
  # intercepts the applications closure and generates an event to the form requesting
  # closure.  Allows the form to control closure.  If the form approves it'll send
  # back an event closing the application
  #`
  def closeTrap(self,event):
    object = _eventObjTowxWindow(event)
    if event.CanVeto():
      self.dispatchEvent(events.Event('requestEXIT'))
    else:
      object.Destroy()

  # Called by SplashScreen when timer expires
  def onCloseSplash(self):
    # If the splashscreen has timed out before the form is
    # ready then go ahead and display what we've got.  This
    # is a desperate attempt to eliminate the vile
    #
    # Gdk-ERROR **: BadWindow (invalid Window parameter)
    #  serial 1228 error_code 3 request_code 15 minor_code 0
    #
    # on heavily loaded machines.  The key line seems to be
    # wxYield() but I'm leaving the others there because I
    # can :P  The fix works for me all the way down to a
    # splash duration=1 millisecond
    #
    self._wxapp.SetTopWindow(self.mainWindow)
    self.mainWindow.Show(true)
    wxYield() # Flush the pending queue to insure something is
              # there before the splashscreen dies
                
    # For some reason, a simple Close() doesn't cooperate with
    # the modal login box and all hell breaks loose
    self.splash.Show(0)
    self.splash.Destroy()


#############################################################################
#############################################################################
#
# UI Objects
#
# A set of standard UI objects that allow us to move more code into UIbase
# These map generic functionality to the specific UI widget set in use by
# this driver.
#
#############################################################################
#############################################################################

#
# UIHelper
#
# Provides a UI widget set of std functions to reduce the number of functions
# require in each UIclass listed below
class UIHelper:
  def showModal(self):
    self.ShowModal()

  def show(self, flag=TRUE):
    self.Show(flag)

  def destroy(self):
    self.Destroy()

  def indexedFocus(self, index):
    self.widgets[index].SetFocus()

  def setValue(self, value, index=0, enabled=1):
    # These must be here or dropdown style controls
    # will get events they shouldn't and break.
    widget = self.widgets[index]
    widget.SetEvtHandlerEnabled(FALSE)


    # Check if foreign key changed
    object = _IdToGFObj[widget.GetId()]
    try:
      if object.style == "dropdown" and \
          not object._allowedValues == widget._origAllowedValues:
        try:
          # Not in wx 2.2.x
          widget.Freeze()
        except AttributeError:
          pass
        widget._origAllowedValues = object._allowedValues
        widget.Clear()
        for value in object._allowedValuesDescr:
          widget.Append(value)
        try:
          # Not in wx 2.2.x
          widget.Thaw()
        except AttributeError:
          pass
    except AttributeError:
      pass

    widget.SetValue(value)
    widget.Enable(enabled)
    widget.SetEvtHandlerEnabled(TRUE)
    if enabled:
      widget.SetBackgroundColour(widget.__origBackgroundColor)
    else:
      widget.SetBackgroundColour(_disabledColour)
    widget.Refresh()

  def setCursorPosition(self, position, index=0):
    try:
      self.widgets[index].SetInsertionPoint(position)
    except AttributeError:
      pass  # For label-style entries

  def setSelectedArea(self, selection1, selection2, index=0):
    try:
      self.widgets[index].SetSelection(selection1, selection2)
    except (AttributeError, TypeError):
      pass  # For label-style & dropdown entries


  def _addToCrossRef(self, widget,gfobject, uiobject):
    id = widget.GetId()
    _IdToWxObj[id]=widget
    _IdToGFObj[id]=gfobject
    _IdToUIObj[id]=uiobject
    widget.__origBackgroundColor = widget.GetBackgroundColour()


  def _deleteFromCrossRef(self, widget, object):
    id = widget.GetId()
    try:
      del _IdToWxObj[id]
      del _IdToGFObj[id]
      del _IdToUIObj[id]
    except:
      pass


  def createWidget(self, event, spacer):

    newWidget = self._createWidget(event, spacer)
    initFont(newWidget)
    self._addToCrossRef(newWidget,event.object,self)
    return newWidget

  def cleanup(self, object):
    for widget in self.widgets[:]:
      id = widget.GetId()
      del _IdToWxObj[id]
      del _IdToGFObj[id]
      del _IdToUIObj[id]
      self.widgets.pop(0)
      self._deleteFromCrossRef(widget, object)


#
# UILabel
#
# Widget set specific function that creates a single instance of a label
#
class UILabel(UIHelper, UIWidget):
  alignmentStyle = { 'left'  : wxALIGN_LEFT|wxST_NO_AUTORESIZE,
                     'center': wxALIGN_CENTRE|wxST_NO_AUTORESIZE,
                     'right' : wxALIGN_RIGHT|wxST_NO_AUTORESIZE,
                     }

  def _createWidget(self, event, spacer):
    object = event.object
    style = self.alignmentStyle[object.alignment]
    newWidget = wxStaticText(event.container, -1, str(object.text),
                             wxPoint(object.x*event.widgetWidth,
                                     (object.y+spacer+(spacer*object._gap))*event.widgetHeight),
                             wxSize(self.itemWidth,self.itemHeight), style)
    return newWidget

#
# UIBox
#
# Widget set specific function that creates a single instance of a border box
#
class UIBox(UIHelper, UIWidget):
  def _createWidget(self, event, spacer):

    object = event.object

    # this panel makes the GFBox be a container
##
## TODO: Commented out prior to 0.4.0 release
##    newPanel = wxPanel(event.container,-1,
##                       wxPoint(object.x*event.widgetWidth+(event.widgetWidth/2),
##                               (object.y+spacer+(spacer*object._gap))*event.widgetHeight+(event.widgetHeight/2)),
##                       wxSize((object.width-1)*event.widgetWidth,
##                              (object.height-1)*event.widgetHeight))
##    newWidget = wxStaticBox(newPanel,-1,str(object.label),
##                       wxPoint(0,0),
    newWidget = wxStaticBox(event.container,-1,str(object.label),
                       wxPoint(object.x*event.widgetWidth+(event.widgetWidth/2),
                               (object.y+spacer+(spacer*object._gap))*event.widgetHeight+(event.widgetHeight/2)),
                       wxSize((object.width-1)*event.widgetWidth,
                               (object.height-1)*event.widgetHeight))
##    return newPanel
    return newWidget

#
# UIScrollBar
#
# Widget set specific function that creates a single instance of a scroll bar used
# to navigate multiple records
#
class UIScrollBar(UIHelper, UIWidget):
  def _createWidget(self, event, spacer):
    object = event.object
    newWidget =wxScrollBar(event.container,-1,
                           wxPoint(object.x*event.widgetWidth,(object.y+spacer)*event.widgetHeight),
                           wxSize(object.width*event.widgetWidth,object.height*event.widgetHeight),
                           wxSB_VERTICAL)
    return newWidget

#
# UIButton
#
# Widget set specific function that creates a single instance of a button
#
class UIButton(UIHelper, UIWidget):
  def _createWidget(self, event, spacer):
    object = event.object
    newWidget = wxButton(event.container,-1,str(object.label),
                         wxPoint(object.x*event.widgetWidth,(object.y+spacer)*event.widgetHeight),
                         wxSize(object.width*event.widgetWidth,object.height*event.widgetHeight)
                         )
    if event.initialize:
      self._eventHandler = event.eventHandler
      EVT_BUTTON(newWidget,newWidget.GetId(),self.buttonHandler)
      _setDefaultEventHandlers(newWidget, event.eventHandler, event.initialize)

    return newWidget

  def buttonHandler(self,event):
    if event.GetEventType() == wxEVT_COMMAND_BUTTON_CLICKED:
      action = None
      object = _eventObjTowxWindow(event)

      id = object.GetId()
      gfObject     = _IdToGFObj[id]
      action = events.Event('fireTRIGGER',gfObject.trigger)
      self._eventHandler(action)


#
# UIPage
#
# Widget set specific function that creates a single instance of a page
#
# Note: The event trap for changing notebook pages is attached to the UIForm
#
class UIPage(UIHelper, UIWidget):
  def _createWidget(self, event, spacer):
    object = event.object
    if event.interface._notebook:
      newWidget = wxPanel(event.interface._notebook, -1,wxDefaultPosition,event.interface._notebook.GetClientSize())
      event.interface._notebook.AddPage(newWidget,hasattr(object,'caption') and object.caption or object.name)
      newWidget.Show(1)
    else:
      newWidget = wxPanel(event.container, -1,wxDefaultPosition,event.container.GetSize())
      newWidget.Show(0)

    _setDefaultEventHandlers(newWidget, event.eventHandler, event.initialize)

    event.interface._pageList.append(newWidget)
    return newWidget

#
# UIEntry
#
# Widget set specific function that creates a single instance of a data entry widget
#
class UIEntry(UIHelper, UIWidget):
  def _createWidget(self, event, spacer):
    object = event.object

    style = object.style
    defaultPoint =  wxPoint(object.x*event.widgetWidth,
                            (object.y+spacer+(object._gap*spacer))*event.widgetHeight)
    defaultSize  = wxSize(self.itemWidth,self.itemHeight+1)

    if style == 'dropdown':
      if event.initialize:
        choices = object.allowedValues()[1]
      else:
        choices = [""]

      newWidget = wxComboBox(event.container, -1, "",defaultPoint,
                              wxSize(self.itemWidth,self.itemHeight),
                             choices, wxCB_DROPDOWN,) # |wxWANTS_CHARS,)
      newWidget._origAllowedValues = object._allowedValues

      newWidget.SetValue("")
      EVT_COMBOBOX(newWidget, newWidget.GetId(), self.comboHandler)


    elif style == 'label':
      newWidget = wxStaticText(event.container, -1, "",defaultPoint,defaultSize,
                               wxST_NO_AUTORESIZE)
      # To make the label consistent with the other entry styles...
      newWidget.SetValue = newWidget.SetLabel
      newWidget.GetValue = newWidget.GetLabel

    elif style == 'checkbox':
      newWidget = wxCheckBox(event.container, -1, "",defaultPoint,
                             wxSize(2*event.textWidth,event.textHeight),
                             wxNO_BORDER)
      newWidget.SetValue(0)
      EVT_CHECKBOX(newWidget, newWidget.GetId(), self.checkboxHandler)

    else: # Normal text box
      if event.initialize:
        styles = wxTE_PROCESS_TAB
        if object.height > 1:
          styles = styles|wxTE_MULTILINE
        value = ""  ## object.getValue()
      else:
        styles = 0
        value = ""

      newWidget = wxTextCtrl(event.container, -1, value, defaultPoint, defaultSize, styles)
    self._eventHandler = event.eventHandler
    _setDefaultEventHandlers(newWidget, event.eventHandler, event.initialize)
    return newWidget

  def comboHandler(self, event):
    action = None
    if event.GetEventType() == wxEVT_COMMAND_COMBOBOX_SELECTED:
      object = _eventObjTowxWindow(event)
      id = object.GetId()

      selection = event.GetSelection()

      gfObject     = _IdToGFObj[id]
      eventdata = [gfObject, event.GetString()]
      action = events.Event('requestREPLACEVALUE',object=gfObject,
                              index=selection, text=event.GetString())
    self._eventHandler(action)

  def checkboxHandler(self,event):
    if event.GetEventType() == wxEVT_COMMAND_CHECKBOX_CLICKED:
      object = _eventObjTowxWindow(event)
      id = object.GetId()
      
      gfObject     = _IdToGFObj[id]
      eventdata = [gfObject, event.GetString()]
      action = events.Event('requestTOGGLECHKBOX')
      self._eventHandler(action)

#
# UIForm
#
# Widget set specific function that creates a single instance of a Form widget
#
class UIForm(UIHelper, UIWidget):
  tabStyles = {'left':wxNB_LEFT,
               'right':wxNB_RIGHT,
               'bottom':wxNB_BOTTOM,
               'top':0,
               }

  def _createWidget(self, event, spacer):
    object = event.object
    self.__ui = event.ui
    self.__ui.__notebookLoopTrap = 0

    if object.tabbed:
      tabstyle =self.tabStyles[object.tabbed]
      newWidget = wxNotebook(event.container,-1,wxPoint(10,10),
                             size=event.interface.mainWindow.panel.GetClientSize(),
                             style=tabstyle)
      newWidget.Show(TRUE)
      EVT_NOTEBOOK_PAGE_CHANGING(newWidget, newWidget.GetId(), self.notebookTabHandler)

      event.interface._notebook  = newWidget
    else:
      newWidget = wxPanel(event.interface.mainWindow.panel,-1, wxDefaultPosition)
      event.interface._notebook = None

    if event.initialize:
      EVT_CLOSE(event.interface.mainWindow,event.interface.closeTrap)

    self._eventHandler = event.eventHandler
    return newWidget

  def notebookTabHandler(self,event):
    if event.GetEventType() == wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING:
      action = None

      if self.__ui.__notebookLoopTrap == 0:
        action = events.Event('requestPAGE',event.GetSelection())
        self.__ui.__notebookLoopTrap = 1
      else:
        self.__ui.__notebookLoopTrap = 0

    if action:
      self._eventHandler(action)

#####################################################################
##
## Required Dialog box support
##
#####################################################################

#
# UIAbout
#
# A dialog box providing about info
#
class UIAbout(wxMessageDialog, UIHelper):
  def __init__(self, interface, programVersion, formName, formVersion, author, description):
    text = _("GNUE Forms")+"\n"+      \
    _("  Version : ")+"%s\n"+         \
    _("  Driver  : UIwxpython")+"\n"+ \
    _("-= Form Info =-")+"\n"+        \
    _("  Name   : ")+"%s\n"+          \
    _("  Version: ")+"%s\n"+          \
    _("  Author : ")+"%s\n"+          \
    _("  Description:")+"%s\n"
    wxMessageDialog.__init__(self, interface,
                             text % (programVersion, formName ,formVersion,author,description),
                             _("About"), wxOK | wxICON_INFORMATION)
#
# UImessageBox
#
# A dialog box providing basic message info
#
class UIMessageBox(wxMessageDialog, UIHelper):
  def __init__(self, interface, message, caption):
    message = lineWrap(message,60)
    wxMessageDialog.__init__(self, interface, message, caption, style=wxOK|wxICON_EXCLAMATION)

#####################################################################
##
## Keymapper Support
##
#####################################################################
from gnue.forms.GFKeyMapper import vk

# Translate from wx keystrokes to our virtual keystrokes
wxKeyTranslations = {
   vk.F1     : WXK_F1,        vk.F2        : WXK_F2,
   vk.F3     : WXK_F3,        vk.F4        : WXK_F4,
   vk.F5     : WXK_F5,        vk.F6        : WXK_F6,
   vk.F7     : WXK_F7,        vk.F8        : WXK_F8,
   vk.F9     : WXK_F9,        vk.F10       : WXK_F10,
   vk.F11    : WXK_F11,       vk.F12       : WXK_F12,
   vk.INSERT : WXK_INSERT,    vk.DELETE    : WXK_DELETE,
   vk.HOME   : WXK_HOME,      vk.END       : WXK_END,
   vk.PAGEUP : WXK_PRIOR,     vk.PAGEDOWN  : WXK_NEXT,
   vk.UP     : WXK_UP,        vk.DOWN      : WXK_DOWN,
   vk.LEFT   : WXK_LEFT,      vk.RIGHT     : WXK_RIGHT,
   vk.TAB    : WXK_TAB,
   vk.ENTER  : WXK_RETURN,    vk.BACKSPACE : WXK_BACK }

GFKeyMapper.KeyMapper.setUIKeyMap(wxKeyTranslations)

#####################################################################
##
## Basic Event Processing
##
#####################################################################

def _setDefaultEventHandlers(newWidget, eventHandler, initialize):
  if initialize:
    # TODO: this should use one instance
    newWidget.PushEventHandler(mouseEvtHandler(eventHandler))
    newWidget.PushEventHandler(keyboardEvtHandler(eventHandler))

class uiBaseEvtHandler(wxEvtHandler):
  def __init__(self,eventList,eventProcessor):
    wxEvtHandler.__init__(self)
    self._eventProcessor = eventProcessor
    for eventType in eventList:
      self.Connect(-1, -1, eventType,self._processEvent)

  def _processEvent(self,event):
    GDebug.printMesg(0,"uiBaseEvtHandler _processEvent was called?!?")

class mouseEvtHandler(uiBaseEvtHandler):
  def __init__(self,eventProcessor):
    eventList = [wxEVT_LEFT_DOWN]
    uiBaseEvtHandler.__init__(self, eventList, eventProcessor)

  #
  # The putzing about with the event timestamp is to
  # allow us to process mouse click events twice. 
  # The mouse handler is called before the cursor 
  # position is updated so GetInsertionPoint returns 
  # the position from the widget losing focus.
  # setting the timestamp to zero is a flag to prevent
  # infinate loops and allow us to avoid process that does 
  # not need done twice.

  def _processEvent(self,event):
    global _charWidth,_charHeight
    object = _eventObjTowxWindow(event)

    id = object.GetId()
    gfObject     = _IdToGFObj[id]
    screenWidget = _IdToWxObj[id]
    count        = _IdToUIObj[id].widgets.index(screenWidget)

    if event.GetTimestamp():
      self._eventProcessor(events.Event('requestFOCUS',gfObject))
      self._eventProcessor(events.Event('requestJUMPRECORD',count - gfObject._visibleIndex))

      if isinstance(screenWidget,wxTextCtrl):
        event.SetTimestamp(0)
        self.AddPendingEvent(event)

    else:
      textObject= wxPyTypeCast(object, 'wxTextCtrl')
      cursorPosition = textObject.GetInsertionPoint()
      self._eventProcessor(events.Event('requestCURSORMOVE',position=cursorPosition))

    #if isinstance(screenWidget,wxCheckBox):
    #  self._eventProcessor(events.Event('requestTOGGLECHKBOX'))
      
    event.Skip()
    #  if isinstance(screenWidget,wxTextCtrl):
    #    event.SetTimestamp(0)
    #    self.AddPendingEvent(event)
    #else:
    #  textObject= wxPyTypeCast(object, 'wxTextCtrl')
    #  cursorPosition = textObject.GetInsertionPoint()
    #  print cursorPosition

#    if gfObject._type == 'GFButton':
#      self._eventProcessor(events.Event('fireTRIGGER',gfObject.trigger))



#
# keyboardEvtHandler
#
# Note: wxEVT_KEY_DOWN event seems to only return uppercase chars so
#       i don't trap it.
#

class keyboardEvtHandler(uiBaseEvtHandler):
  def __init__(self, eventProcessor):
    eventList = [wxEVT_CHAR]
    uiBaseEvtHandler.__init__(self, eventList, eventProcessor)

  def _processEvent(self,event):
    action = None
    # hack for swig shortcomming needed by wxPython 2.2.x (2.3 does not need this)
    # compute the location of the character in the widget
    object = _IdToGFObj[_eventObjTowxWindow(event).GetId()]

    keycode = event.KeyCode()

    #
    # Sigh... a hack for using <enter> in multiline entries
    #
    if  keycode == 13 and \
        not event.ShiftDown() and \
        not event.ControlDown() and \
        not event.AltDown() and \
        int (gConfigForms('enterIsNewLine')) and \
        object.height > 1:

      command = 'NEWLINE'

    else:


      #
      # Get the event to process from the KeyMapper
      #
      command = GFKeyMapper.KeyMapper.getEvent(
        keycode,
        event.ShiftDown(),
        event.ControlDown(),
        event.AltDown())

    if command == 'JUMPRECORD':
      global _PROMPTFORRECORD
      action = _PROMPTFORRECORD()

    elif command == 'NEWLINE':
      action = events.Event('requestKEYPRESS', '\n',
                     text='\n',
                     code=10)

    elif command:
      action = events.Event('request%s' % command)

    elif object._type == 'GFButton':
      action = events.Event('fireTRIGGER',object.trigger)

    elif object.style == 'checkbox' and object._type == 'GFEntry':
      # <space> <=> <click>
      if keycode == 32:
          action = events.Event('requestTOGGLECHKBOX')
      else:
          # maybe some background error message here
          pass
    else:
      try:
        char = chr(keycode)
        if char in string.printable or char == "\n" or \
           128 <= keycode <= 255:
          action = events.Event('requestKEYPRESS', char,
                         text=char,
                         code=keycode)
      except ValueError:
        pass #event.Skip()

    if action:
      self._eventProcessor(action)

#####################################################################
##
## Login Support
##
#####################################################################

#
# UILoginHandler
#
class UILoginHandler(GLoginHandler.LoginHandler):
  def __init__(self):
    self._wxapp = getWxApp()
    self.dlg = None

  def getLogin(self, loginData, errortext=None):#"Invalid username/password"):
    if 1: # TODO: This is left here to prevent me from having to unindent this code
          # TODO: Since the UI stuff is currently being gutted I'm not wasting the time
          # TODO: to do the right thing
      if len(loginData[1]):
        loginMesg = _('Login required for\n"%s"') % (loginData[1])
      else:
        loginMesg = _('Login required for %s') % (loginData[0])

      self.dlg = wxDialog(NULL, -1, _("GNU Enterprise: Login to %s") % loginData[0])
      self.dlg.SetAutoLayout(true)

      bmp = wxImage(images_dir+gConfigForms('smallPNG'), wxBITMAP_TYPE_PNG).ConvertToBitmap()

      messageField = WrappedStaticText(self.dlg, -1, str(loginMesg), 300, style=wxALIGN_CENTER)

      self.textctrlList = []
      labelList = []

      dlgWidth = max(bmp.GetWidth(), messageField.GetSize().GetWidth()+20)

      dlgHeight = bmp.GetHeight() + messageField.GetSize().GetHeight() + 80

      xSpacing = 0
      ySpacing = 0
      fieldLabelWidth = 0

      for prompt in loginData[2]:
        s = wxStaticText(self.dlg, -1, '%s:' % prompt[1])
        labelList.append(s)
        if prompt[2]:
          t = wxTextCtrl(self.dlg, -1,"",wxPoint(1, 1), wxSize(150, 20),
                         style=wxTE_PASSWORD|wxTE_PROCESS_ENTER)
        else:
          t = wxTextCtrl(self.dlg, -1,"",wxPoint(1, 1), wxSize(150, 20),
                         style=wxTE_PROCESS_ENTER)

        myID = len(self.textctrlList)
        self.textctrlList.append(t)
        EVT_CHAR(t, LoginFieldHandler(self, myID).loginFieldEventTrap)

        fieldLabelWidth = max(fieldLabelWidth,
                  s.GetSize().GetWidth() + t.GetSize().GetWidth() + 10)

        dlgWidth = max(dlgWidth, \
                       s.GetSize().GetWidth() + t.GetSize().GetWidth() + 20)

        xSpacing = max(xSpacing, s.GetSize().GetWidth())
        ySpacing = max(ySpacing, s.GetSize().GetHeight())
        ySpacing = max(ySpacing, t.GetSize().GetHeight())


      loginId = wxNewId()
      cancelId = wxNewId()

      loginButton = wxButton(self.dlg,loginId,_('Login'))
      cancelButton = wxButton(self.dlg,cancelId,_('Cancel'))

      EVT_BUTTON(self.dlg, loginId, self.loginButtonEventTrap)
      EVT_BUTTON(self.dlg, cancelId, self.loginCancelEventTrap)
      dlgWidth = max(dlgWidth, loginButton.GetSize().GetWidth() +
                            cancelButton.GetSize().GetWidth() + 6) + 20

      dlgHeight += max(loginButton.GetSize().GetHeight(),
                       cancelButton.GetSize().GetHeight()) - 6

      if errortext:
        errorField = WrappedStaticText(self.dlg, -1, str(errortext), 300,
                                       style=wxALIGN_CENTER)
        errorField.SetForegroundColour(wxColour(223,0,0))

        dlgWidth = max(dlgWidth, errorField.GetSize().width+10)
        dlgHeight += errorField.GetSize().height + 6


      firstY = bmp.GetHeight() + messageField.GetSize().GetHeight() + 50
      lastY = firstY
      xSpacing += 10	# Add whitespace between widgets
      ySpacing += 6	# Add whitespace between widgets
      xPos = dlgWidth/2 - fieldLabelWidth/2


      # Move the fields and labels into position
      for i in range(0, len(self.textctrlList)):
        dlgHeight = dlgHeight + ySpacing
        labelList[i].SetPosition(wxPoint(xPos, lastY))
        self.textctrlList[i].SetPosition(wxPoint(xPos + xSpacing, lastY))
        lastY = lastY + ySpacing

      if errortext:
        errorField.SetPosition(
          wxPoint(dlgWidth/2 - errorField.GetSize().width/2,
                lastY+3))

      # Set the focus to the first text entry field
      self.textctrlList[0].SetFocus()

      # Create and position the logo
      wxStaticBitmap(self.dlg,-1, bmp,
                     wxPoint((dlgWidth-bmp.GetWidth())/2, 12),
                     wxSize(bmp.GetWidth(), bmp.GetHeight()))

      # Move the various widgets into position
      messageField.SetPosition(
        wxPoint(dlgWidth/2 - messageField.GetSize().GetWidth()/2,
                30 + bmp.GetHeight()))

      cancelButton.SetPosition(
        wxPoint(dlgWidth - 10 - cancelButton.GetSize().GetWidth(),
                dlgHeight - 10 - max(loginButton.GetSize().GetHeight(),
                                            cancelButton.GetSize().GetHeight())))
      loginButton.SetPosition(
        wxPoint(dlgWidth - 16 - cancelButton.GetSize().GetWidth() - \
                loginButton.GetSize().GetWidth(),
                dlgHeight - 10 - max(loginButton.GetSize().GetHeight(),
                                            cancelButton.GetSize().GetHeight())))

      self.loginButton = loginButton

      self.dlg.SetSize(wxSize(dlgWidth, dlgHeight))

      self.dlg.Refresh()
      self.dlg.Fit()
      self.dlg.Raise()
      self.dlg.CenterOnScreen()

    # If user cancels, this will be set to 0
    self._completed = 0
    self.dlg.ShowModal()

    if not self._completed:
      raise GLoginHandler.UserCanceledLogin

    rv = {}
    for i in range(0, len(loginData[2])):
      rv[loginData[2][i][0]] = self.textctrlList[i].GetValue()

    return rv

  #
  # Login is completed, for whatever reason
  #
  def loginCompleted(self, successful):
    self._completed = successful
    self.dlg.EndModal(1)

  #
  # Called when user clicks "login"
  #
  def loginButtonEventTrap(self, event):
    self.loginCompleted(1)

  #
  # Called when user clicks "cancel"
  #
  def loginCancelEventTrap(self, event):
    self.loginCompleted(0)

  #
  # TODO: This is a hack required because windows
  # TODO: seems to have issues with wxWindows dialogs
  #
  def destroyLoginDialog(self):
    self.dlg.Destroy()

#
# LoginFieldHandler
#
# Used by the login handler
# enables the user to press return and have it jump to the next box
#
class LoginFieldHandler:
  def __init__(self, app, seq):
    self.app = app
    self.seq = seq

  def loginFieldEventTrap(self, event):
     if event.KeyCode() in (WXK_RETURN, WXK_TAB):
       if self.seq < len(self.app.textctrlList) - 1:
         self.app.textctrlList[self.seq+1].SetFocus()
       else:
         if event.KeyCode() == WXK_TAB:
           if event.ShiftDown():
             self.app.textctrlList[self.seq-1].SetFocus()
           else:
             self.app.loginButton.SetFocus()
         else:
           self.app.loginCompleted(1)
     else:
      event.Skip()



class WrappedStaticText(wxStaticText):
  def __init__(self, parent, id, label, width, *args, **params):
    wxStaticText.__init__(self, parent, id, "bah!", *args, **params)


    textSoFar = ""
    thisLine = ""
    for part in string.split(label,'\n'):
      for word in string.split(part):
        self.SetLabel(thisLine + word)
        if self.GetSize().width > width:
          textSoFar += thisLine + " \n"
          thisLine = word + " "
        else:
          thisLine += word + " "

      textSoFar += thisLine + " \n"
      thisLine = ""

    if len(textSoFar):
      self.SetLabel(string.replace(textSoFar,' \n','\n')[:-1])
    else:
      self.SetLabel("")





# Needed by Designer
WIDGETS = {'GFLabel'     : UILabel,
           'GFBox'       : UIBox,
           'GFPage'      : UIPage,
           'GFForm'      : UIForm,
           'GFEntry'     : UIEntry,
           'GFButton'    : UIButton,
           'GFScrollBar' : UIScrollBar,
          }


# Shortcut
images_dir = GConfig.getInstalledBase('forms_images','common_images') + '/'


